require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

describe Emvoicer::Invoice do
  context "in general" do
    let(:service) do
      Emvoicer::Services::Base.new
    end

    let(:invoice) do
      attributes = {
        :id => 1,
        :number => "100001",
        :total_amount => 20.0,
        :due_amount => 10.0,
        :issued_on => Date.new(2011, 3, 13),
        :due_on => Date.new(2011, 3, 28),
        :currency => "USD",
        :client_id => 1
      }

      Emvoicer::Invoice.new(service, attributes)
    end

    describe ".service" do
      it "returns service which invoice was created with" do
        invoice.service.should eq(service)
      end
    end

    [
      :id,
      :number,
      :total_amount,
      :due_amount,
      :issued_on,
      :due_on,
      :currency,
      :client_id
    ].each do |attr|
      describe ".#{ attr }" do
        it "returns value of #{ attr } attribute" do
          invoice.send(attr).should eq(invoice.attributes[attr])
        end
      end
    end

    describe ".client(callback)" do
      it "fetches client and calls callback" do
        callback = proc {}
        service.expects(:client).with(invoice.client_id, callback)
        invoice.client(callback)
      end
    end
  end
end
