require File.expand_path(File.dirname(__FILE__) + "/../spec_helper")

collection_xml = File.read(File.expand_path(File.dirname(__FILE__) + "/../fixtures/invoice_collection.xml"))

describe Emvoicer::Services::Blinksale do
  let(:blinksale) do
    Emvoicer::Services::Blinksale.new("test-host", "test-username", "test-password")
  end

  describe ".auth_params" do
    it "returns proper API credentials array" do
      blinksale.auth_params.should eq(["test-username", "test-password"])
    end
  end

  describe ".headers" do
    it "returns proper Accept and ContentType headers" do
      blinksale.headers.should eq({
        "Content-Type" => "application/vnd.blinksale+xml",
        "Accept" => "application/vnd.blinksale+xml"
      })
    end
  end

  describe "#test_authentication_response" do
    context "when response is successful" do
      it "returns true" do
        stub_request(:get, "http://test-username:test-password@test-host.blinksale.com:443/invoices").
          with(:headers => {"Accept"=>"application/vnd.blinksale+xml", "Content-Type"=>"application/vnd.blinksale+xml", "User-Agent"=>"Ruby"}).
          to_return(:status => 200, :body => "", :headers => {})
        
        blinksale.authentication_valid?.should be_true
      end
    end

    context "when response is not successful" do
      it "returns false" do
        stub_request(:get, "http://test-username:test-password@test-host.blinksale.com:443/invoices").
          with(:headers => {"Accept"=>"application/vnd.blinksale+xml", "Content-Type"=>"application/vnd.blinksale+xml", "User-Agent"=>"Ruby"}).
          to_return(:status => 401, :body => "", :headers => {})
          
        blinksale.authentication_valid?.should be_false
      end
    end
  end

  describe "invoice handling" do
    describe ".invoices_url" do
      it "returns proper API invoices url" do
        blinksale.invoices_url.should eq("https://test-host.blinksale.com/invoices")
      end
    end

    describe "#extract_invoice_nodes(source)" do
      it "returns array of nodes to parse" do
        nodes = Emvoicer::Services::Blinksale.extract_invoice_nodes(collection_xml)
        nodes.should have(2).items
        nodes.each { |n| n.should be_an_instance_of(Nokogiri::XML::Element) }
      end
    end

    describe "#parse_invoice(nokogiri_element)" do
      let(:nokogiri_element) do
        Nokogiri::XML(collection_xml).xpath("//xmlns:invoice").first
      end

      it "returns hash with proper invoice attributes" do
        hash = Emvoicer::Services::Blinksale.parse_invoice(nokogiri_element)
      
        hash[:id].should eql(1)
        hash[:number].should eql("100001")
        hash[:total_amount].should eql(20.0)
        hash[:due_amount].should eql(10.0)
        hash[:issued_on].should eql(Date.new(2011, 3, 13))
        hash[:due_on].should eql(Date.new(2011, 3, 28))
        hash[:currency].should eql("USD")

        hash[:client_id].should eql(1)
      end
    end
  end
end
